@@ -134,6 +134,8 @@ SeasDummy <- function(x)
 # Largely based on uroot package code for CH.test()
 SD.test <- function (wts, s=frequency(wts))
 {
+    if(any(is.na(wts)))
+        stop("Series contains missing values. Please choose order of seasonal differencing manually.")
     if(s==1)
         stop("Not seasonal data")
     t0 <- start(wts)
@@ -143,10 +145,10 @@ SD.test <- function (wts, s=frequency(wts))
     frec <- rep(1, as.integer((s+1)/2))
     ltrunc <- round(s * (N/100)^0.25)
     R1 <- as.matrix(SeasDummy(wts))
-    lmch <- lm(wts ~ R1)   # run the regression : y(i) = mu+f(i)'gamma(i)+e(i)
+    lmch <- lm(wts ~ R1,na.action=na.exclude)   # run the regression : y(i) = mu+f(i)'gamma(i)+e(i)
     Fhat <- Fhataux <- matrix(nrow = N, ncol = s-1)
     for (i in 1:(s-1))
-        Fhataux[, i] <- R1[,i] * lmch$residuals
+        Fhataux[, i] <- R1[,i] * residuals(lmch)
     for (i in 1:N)
     {
         for (n in 1:(s - 1))
